@@ -3,8 +3,11 @@
 #'
 #' @export
 #' @template args
+#' @template r6
 #' @param path URL path, appended to the base URL
-#' @param query query terms, as a named list
+#' @param query query terms, as a named list. any numeric values are
+#' passed through [format()] to prevent larger numbers from being
+#' scientifically formatted
 #' @param body body as an R list
 #' @param encode one of form, multipart, json, or raw
 #' @param disk a path to write to. if NULL (default), memory used.
@@ -26,7 +29,7 @@
 #' If you don't pass in a curl handle to the `handle` parameter,
 #' it gets created when a HTTP verb is called. Thus, if you try to get `handle`
 #' after creating a `HttpClient` object only passing `url` parameter, `handle`
-#' will be `NULL`. If you pass a curl handle to the `handle parameter, then
+#' will be `NULL`. If you pass a curl handle to the `handle` parameter, then
 #' you can get the handle from the `HttpClient` object. The response from a
 #' http verb request does have the handle in the `handle` slot.
 #'
@@ -43,7 +46,7 @@
 #'
 #' @examples \dontrun{
 #' # set your own handle
-#' (h <- handle("https://httpbin.org"))
+#' (h <- handle("https://hb.opencpu.org"))
 #' (x <- HttpClient$new(handle = h))
 #' x$handle
 #' x$url
@@ -58,7 +61,7 @@
 #'
 #' # if you just pass a url, we create a handle for you
 #' #  this is how most people will use HttpClient
-#' (x <- HttpClient$new(url = "https://httpbin.org"))
+#' (x <- HttpClient$new(url = "https://hb.opencpu.org"))
 #' x$url
 #' x$handle # is empty, it gets created when a HTTP verb is called
 #' (r1 <- x$get('get'))
@@ -95,7 +98,7 @@
 #'
 #' # query params are URL encoded for you, so DO NOT do it yourself
 #' ## if you url encode yourself, it gets double encoded, and that's bad
-#' (x <- HttpClient$new(url = "https://httpbin.org"))
+#' (x <- HttpClient$new(url = "https://hb.opencpu.org"))
 #' res <- x$get("get", query = list(a = 'hello world'))
 #'
 #' # access intermediate headers in response_headers_all
@@ -341,7 +344,7 @@ HttpClient <- R6::R6Class(
     #' @param verb an HTTP verb supported on this class: "get",
     #' "post", "put", "patch", "delete", "head". Also supports retry.
     #' @examples \dontrun{
-    #' (x <- HttpClient$new(url = "https://httpbin.org"))
+    #' (x <- HttpClient$new(url = "https://hb.opencpu.org"))
     #' x$verb('get')
     #' x$verb('GET')
     #' x$verb('GET', query = list(foo = "bar"))
@@ -383,7 +386,7 @@ HttpClient <- R6::R6Class(
     #' Note that the time spent in the function effectively adds to the wait time,
     #' so it should be kept simple.
     #' @examples \dontrun{
-    #' x <- HttpClient$new(url = "https://httpbin.org")
+    #' x <- HttpClient$new(url = "https://hb.opencpu.org")
     #'
     #' # retry, by default at most 3 times
     #' (res_get <- x$retry("GET", path = "status/400"))
@@ -451,7 +454,7 @@ HttpClient <- R6::R6Class(
     #' parameters; body and any curl options don't change the URL
     #' @return URL (character)
     #' @examples
-    #' x <- HttpClient$new(url = "https://httpbin.org")
+    #' x <- HttpClient$new(url = "https://hb.opencpu.org")
     #' x$url_fetch()
     #' x$url_fetch('get')
     #' x$url_fetch('post')
@@ -520,11 +523,15 @@ HttpClient <- R6::R6Class(
         headers <- list()
       } else {
         hh <- rawToChar(resp$headers %||% raw(0))
+        if (!validEnc(hh)) {
+          Encoding(hh) <- "latin1"
+          if (!validEnc(hh)) stop("Headers aren't encoded in UTF-8 or Latin1")
+        }
         if (is.null(hh) || nchar(hh) == 0) {
           headers <- list()
         } else {
           headers <- lapply(curl::parse_headers(hh, multiple = TRUE),
-            headers_parse)
+            head_parse)
         }
       }
       # build response
